/**
 * Created by John on 7/5/2017.
 * Modified by TFG on 7/21/2020.
 *
 * Common logic for manipulating Loader models
 *
 * CAUTION !  each load now has a common starting base :
 *    csv_line[0] -> transactionId
 *    csv_line[1] -> file name
 *    csv_line[2] -> expected number of messages
 */

// CAUTION !
// call from this module the common functions without passing the app or system settings, it will not work
// app or winston or logger will cause the common function just to don't work, NO ERRORS ISSUED BY THE SYSTEM !

var express = require('express');
var app = express();
//var app = require("../../child");

var awcblcommon = require('../config/awcblcommon');

var debug = false;               // Flag used  to enable extra log messages for debug/problem solving

// ======================= Row position declaration =========================================================
//
// Modify this area if there is any change in the format of the data coming from the queue !
// CAUTION !!!!!  These queue format are derived from a similar definition in m3-api code !is using these sequences to format the output data for the m3-loader
// THEY MUST BE KEPT IN SYNC IN CASE OF MODIFICATION !!!!
// See the file component/config/awcblvalidation.js on the fresenius-m3-api code !
// Note that these positions include the metadata.

// Master Item position
//  - transactionID
//  - file_name
//  - Number_messages
//  - sap_material
//  - description
//  - cnt_ind
//  - cnt_description
//  - med_vendor     (index, not description, separation is # - not used here!!)
//  - vendor_material
//  - conv
//  - count_type
//  - alternates
//  - item_message
//  - unit_price
//  - vendor_id
//  - onhand_inventory_ind
//  - discontinued
const rowMTPos = {"transactionId":0, "fileName":1, "expNumber":2,
    "sapMaterial":3, "description":4, "cntInd":5, "cntDescription":6, "medVendor":7, "vendorMaterial":8, "conv":9,
    "countType":10, "alternates":11, "itemMessage":12, "unitPrice":13, "vendorId":14, "onHand":15, "discontinued":16};

// Clinic Profile position
//  - transactionID
//  - file_name
//  - Number_messages
//  - clinic_id
//  - test_catalog   (index, not description, separation is # - not used here!!)
//  - Mircera_visible
//  - Mircera_Io
//  - Areanesp_visible
//  - Areanesp_Io
//  - Epogen_Visible
//  - Epogen_Io
//  - Retacrit_visible
//  - Retacrit_Io
//  - Count_each_in
const rowCPPos = {"transactionId":0, "fileName":1, "expNumber":2,
    "clinicId":3, "testCatalog":4, "mirceraVisible":5, "mirceraIo":6, "areanespVisible":7, "areanespIo":8,
    "epogenVisible":9, "epogenIo":10, "retacritVisible":11, "retacritIo":12,
    "countEachIn":13};

// Clinic Routing position
//  - transactionID
//  - file_name
//  - Number_messages
//  - clinicId
//  - vendorId
//  - route
//  - vendorDisplay  (added by validation process)
const rowCRPos = {"transactionId":0, "fileName":1, "expNumber":2,
    "clinicId":3, "vendorId":4, "route":5, "vendorDisplay":6};

// Par Value  position
//  - transactionID
//  - file_name
//  - Number_messages
//  - clinicId
//  - productId
//  - qoh
//  - min_par
//  - max_par
//  - vendorId  (added by validation process)
//  - vendor_display (added by validation process)
//  - productName  (added by validation process)
const rowPVPos = {"transactionId":0, "fileName":1, "expNumber":2,
    "clinicId":3, "vendorMaterial":4, "qoh":5, "minPar":6, "maxPar":7, "vendorId":8, "vendorDisplay":9, "productName":10};

// ============================================================================================================


var moment = require("moment");
if (!String.prototype.startsWith) {
    String.prototype.startsWith = function(searchString, position) {
        position = position || 0;
        return this.indexOf(searchString, position) === position;
    };
}

function LoaderHelper(models, winston) {
    this.models = models;
    this.winston = winston
    this.logger = null;
    if (this.winston) {
        this.logger = this.winston.loggers.get("main");
    } else {
        this.logger = console;
        if (!this.logger.debug) this.logger.debug = console.log;
    }
}

/**
 * Saves a single Clinic Routing  record to the database tbl_vendor_route_clinic via promise
 * IMPORTANT ! It is assumed the data coming here are already vetted by m3-api during the
 * validation !!!!!!
 * It means no specific check on the data are performed here !
 *
 * @param {Object} msg JSON from Http
 */
LoaderHelper.prototype.saveLoaderCR = function(msg) {
    var self = this;

    return new Promise(function (resolve, reject) {
        var inClinicId;                 // Input clinic ID
        var inVendorId;                 // Input vendor ID
        var inPathRoute;                // Input Path route
        var inVendorDisplay;            // Input vendor display

        var csv_line = msg.split(/,(?=(?:(?:[^"]*"){2})*[^"]*$)/);
        csv_line = csv_line || [];

        if (csv_line.length > 1) {
            try {
                inClinicId      = csv_line[rowCRPos.clinicId].split(',').join('');   // Assign input Clinic ID
                inVendorId      = csv_line[rowCRPos.vendorId].split(',').join('');   // Assign input Vendor ID
                inPathRoute     = csv_line[rowCRPos.route].split(',').join('');   // Assign Path route
                // Extra parameter not in the original CSV file
                inVendorDisplay = csv_line[rowCRPos.vendorDisplay].split(',').join('');   // Assign Vendor display name
            } catch (err) {
                // The test is needed in case the CSV record received is just the original one from the
                // bulk load CSV file, without the two extra parameters
                if (inVendorDisplay === undefined) {
                    inVendorDisplay = "--";
                } else {
                    reject("CAUTION !! CR - wrong CSV format received : " + err);
                }
            }

            if (debug === true) {
                self.logger.debug("  Payload retrieved ");
                self.logger.debug("           clinicID : " + inClinicId);  // Clinic ID
                self.logger.debug("           vendorId : " + inVendorId);  // Vendor Id
                self.logger.debug("          pathRoute : " + inPathRoute);  // Path route
                self.logger.debug("      vendorDisplay : " + inVendorDisplay);  // Vendor name
            }

            // TEMP ! Caution on the VendorDisplay

            self.models.VendorRouteClinic.findOrCreate({
                where: {
                    clinicId: inClinicId,
                    vendorId: inVendorId
                }, defaults: {
                    vendorDisplay: inVendorDisplay,
                    vendorPathRoute: inPathRoute
                    // no other parameters for default as the three above
                }
            }).spread(function (data, created) {
                if (!created) {
                    // Update record
//                    data.clinicId = inClinicId;
//                    data.vendorId = inVendorId;
                    data.vendorPathRoute = inPathRoute;

                    data.save().then(function () {
                        if (debug === true) {
                            self.logger.debug("Record updated ");
                        }
                        // Record updated successfully
                    }).catch(function (err) {
                        reject("CR - problem interacting with DB : " + err);
                    });
                } else {
                    // Record created successfully
                    if (debug === true) {
                        self.logger.debug("Record created ");
                    }
                }
                resolve("OK");
            }).catch(function (err) {
                reject("CR Error handling table : " + err);
            });
        } else {
            // Should never happens !!
            reject("CAUTION !! CR - something wrong received ! ");
        }
    });
};

/**
 * Saves a single Par Value record to the database tbl_sap_item_par_value via promise
 * IMPORTANT ! It is assumed the data coming here are already vetted by m3-api during the
 * validation !!!!!!
 * It means no specific check on the data are performed here !
 *
 * @param {Object} msg JSON from Http
 */
LoaderHelper.prototype.saveLoaderPV = function(msg) {
    var self = this;

    return new Promise(function (resolve, reject) {
        var inClinicId;                 // Input clinic ID
        var inProductId;                // Input product ID
        var inQoh;                      // Input QOH value
        var inMinParValue;              // Input Min par value
        var inMaxParValue;              // Input Max par value
        var inVendorId;                 // Input vendor ID
        var inVendorDisplay;            // Input vendor Display
        var inProductName;              // Input product display name

        var csv_line = msg.split(/,(?=(?:(?:[^"]*"){2})*[^"]*$)/);
        csv_line = csv_line || [];

        if (csv_line.length > 1) {
            // Received something - check if is the information record
            try {
                // Prepare the fields to be loaded in the table reading the received record in the queue
                inClinicId      = csv_line[rowPVPos.clinicId].split(',').join('');   // Assign input Clinic ID
                inProductId     = csv_line[rowPVPos.vendorMaterial].split(',').join('');   // Assign input Product ID
                inQoh           = csv_line[rowPVPos.qoh].split(',').join('');   // Assign input QOH
                inMinParValue   = csv_line[rowPVPos.minPar].split(',').join('');   // Assign input MinParValue
                inMaxParValue   = csv_line[rowPVPos.maxPar].split(',').join('');   // Assign input MaxParValue
                // Extra parameters not in the original CSV file
                inVendorId      = csv_line[rowPVPos.vendorId].split(',').join('');   // Assign input vendorId
                inVendorDisplay = csv_line[rowPVPos.vendorDisplay].split(',').join('');   // Assign input Product Name
                inProductName   = csv_line[rowPVPos.productName].split(',').join('');   // Assign input Product Name
            } catch (err) {
                // The test is needed in case the CSV record received is just the original one from the
                // bulk load CSV file, without the two extra parameters
                if (inVendorId === undefined || inProductName === undefined) {
                    inVendorId = "--";
                    inProductName = "--";
                } else {
                    reject("CAUTION !! PV - wrong CSV format received : " + err);
                }
            }

            if (debug === true) {
                self.logger.debug("  Payload retrieved ");
                self.logger.debug("           clinicID : " + inClinicId);  // Clinic ID
                self.logger.debug("          productId : " + inProductId);  // Product ID
                self.logger.debug("                QOH : " + inQoh);  // QOH
                self.logger.debug("      min Par value : " + inMinParValue);  // Min
                self.logger.debug("      max Par value : " + inMaxParValue);  // Max
                self.logger.debug("           vendorId : " + inVendorId);  // Vendor Id
                self.logger.debug("      vendorDisplay : " + inVendorDisplay);  // Vendor display
                self.logger.debug("        productName : " + inProductName);  // Product Name
            }

            self.models.VendorSapParValue.findOrCreate({
                where: {
                    vendorMaterial: inProductId,
                    vendorId: inVendorId,
                    clinicId: inClinicId
                }, defaults: {
                    qohValue: inQoh,
                    minParValue: inMinParValue,
                    maxParValue: inMaxParValue,
                    description: inVendorDisplay,
                    description: inProductName
                }
            }).spread(function (data, created) {
                if (!created) {
                    data.qohValue = inQoh;
                    data.minParValue = inMinParValue;
                    data.maxParValue = inMaxParValue;
                    data.vendorDisplay = inVendorDisplay;
                    data.description = inProductName;

                    data.save().then(function () {
                        // Update successfull
                        if (debug === true) {
                            self.logger.debug("Record updated ");
                        }
                    }).catch(function (err) {
                        reject("PV - problem interacting with DB : " + err);
                    });
                } else {
                    // Creation successfull
                    if (debug === true) {
                        self.logger.debug("Record created ");
                    }
                }
                resolve("OK");
            }).catch(function (err) {
                reject("PV Error handling table : " + err);
            });
        } else {
            reject("CAUTION !! PV - something wrong received ! ");
        }
    });
};

/**
 * Saves a single Clinic Profile record to the database tbl_clinic_profile via promise
 * IMPORTANT ! It is assumed the data coming here are already vetted by m3-api during the
 * validation !!!!!!
 * It means no specific check on the data are performed here !
 *
 * @param {Object} msg JSON from Http
 */
LoaderHelper.prototype.saveLoaderCP = function(msg) {
    var self = this;

    return new Promise(function (resolve, reject) {
        var clinicIdId;
        var inClinicId;                 // Input clinic ID
        var inMirceraVisible;
        var inMirceraIo;
        var inAreanespVisible;
        var inAreanespIo;
        var inEpogenVisible;
        var inEpogenIo;
        var inRetacritVisible;
        var inRetacritIo;
        var inCountInEach;
        var inDateModified;

        var csv_line = msg.split(/,(?=(?:(?:[^"]*"){2})*[^"]*$)/);
        csv_line = csv_line || [];

        if (debug === true) {
            self.logger.debug("saveLoaderCP - csv line             : " + JSON.stringify(csv_line));
        }

        // NOTE !
        // The inTestCatalog is ALREADY a list of catalogs ID separated by the character #
        // Validation takes care also of the case of no catalog presents.
        // Has no use here since is used before to create the clinicCatalog list associated to this record

        if (csv_line.length > 1) {
            try {
                // Prepare the fields to be loaded in the table reading the received record in the queue
                inClinicId        = csv_line[rowCPPos.clinicId].split(',').join('');
                inMirceraVisible  = csv_line[rowCPPos.mirceraVisible].split('"').join('');
                inMirceraIo       = csv_line[rowCPPos.mirceraIo].split('"').join('');
                inAreanespVisible = csv_line[rowCPPos.areanespVisible].split('"').join('');
                inAreanespIo      = csv_line[rowCPPos.areanespIo].split('"').join('');
                inEpogenVisible   = csv_line[rowCPPos.epogenVisible].split('"').join('');
                inEpogenIo        = csv_line[rowCPPos.epogenIo].split('"').join('');
                inRetacritVisible = csv_line[rowCPPos.retacritVisible].split('"').join('');
                inRetacritIo      = csv_line[rowCPPos.retacritIo].split('"').join('');
                inCountInEach     = csv_line[rowCPPos.countEachIn].split('"').join('');
                inDateModified    = new Date();
            } catch (e) {
                self.logger.error("Catch error : " + JSON.stringify(e));
                reject("CAUTION !! CP - wrong CSV format received ! ");
            }

            if (debug === true) {
                self.logger.debug("  Payload retrieved ");
                self.logger.debug("           clinicID : " + inClinicId);
                self.logger.debug("   inMriceraVisible : " + inMirceraVisible);
                self.logger.debug("        inMriceraIo : " + inMirceraIo);
                self.logger.debug("  inAreanespVisible : " + inAreanespVisible);
                self.logger.debug("       inAreanespIo : " + inAreanespIo);
                self.logger.debug("    inEpogenVisible : " + inEpogenVisible);
                self.logger.debug("         inEpogenIo : " + inEpogenIo);
                self.logger.debug("  inRetacritVisible : " + inRetacritVisible);
                self.logger.debug("       inRetacritIo : " + inRetacritIo);
                self.logger.debug("      inCountInEach : " + inCountInEach);
                self.logger.debug("     inDateModified : " + inDateModified);
            }

            // Building an object with the data received in input
            // Not saving on DB the vendorList (is a virtual field)
            var clinicProfile = self.models.ClinicProfile.build({
                clinicId: inClinicId,
                //vendorList: inTestCatalog,
                mirceraVisible: inMirceraVisible === 'y',
                mirceraIo: inMirceraIo === 'y',
                areanespVisible: inAreanespVisible === 'y',
                areanespIo: inAreanespIo === 'y',
                epogenVisible: inEpogenVisible === 'y',
                epogenIo: inEpogenIo === 'y',
                retacritVisible: inRetacritVisible === 'y',
                retacritIo: inRetacritIo === 'y',
                countInEach: inCountInEach === 'y',
                dateModified: inDateModified
            });

            self.models.ClinicProfile.findOrCreate({
                where: {
                    clinicId: inClinicId
                }, defaults: {
                    clinicProfile
                }
            }).spread(function (data, created) {
                clinicIdId = data.id;      // Save record ID
                if (!created) {
                    data.clinicId = clinicProfile.clinicId;
                    data.mirceraVisible = clinicProfile.mirceraVisible;
                    data.mirceraIo = clinicProfile.mirceraIo;
                    data.areanespVisible = clinicProfile.areanespVisible;
                    data.areanespIo = clinicProfile.areanespIo;
                    data.epogenVisible = clinicProfile.epogenVisible;
                    data.epogenIo = clinicProfile.epogenIo;
                    data.retacritVisible = clinicProfile.retacritVisible;
                    data.retacritIo = clinicProfile.retacritIo;
                    data.countInEach = clinicProfile.countInEach;
                    data.dateModified = clinicProfile.dateModified;

                    data.save().then(function () {
                        if (debug === true) {
                            self.logger.debug("   updated CP record " + data.id);
                        }
                    }).catch(function (err) {
                        reject("CP - problem interacting with DB : " + err);
                    });
                } else {
                    if (debug === true) {
                        self.logger.debug("   created CP record " + data.id);
                    }
                }
                // Bring back the table ID of the saved/updated record !
                resolve(clinicIdId);
            }).catch(function (err) {
                reject("CP Error handling table : " + err);
            });
        } else {
            reject("CAUTION !! CP - something wrong received ! ");
        }
    });
};

/**
 * Remove from the tbl_clinic_catalog all the entries related to a specific clinic ID
 *
 * @param clinic ID ID    (the ID in the tbl_clinic_profile of a clinic ID
 */
LoaderHelper.prototype.removeCPassociated = function(inClinicIdId) {
    var self = this;

    return new Promise(function (resolve, reject) {
        if (debug === true) {
            self.logger.debug("removeCPassociated for " + inClinicIdId);
        }

        // Remove from the table all entries related to the clinicId received
        self.models.ClinicCatalog.destroy({
            where: {
                clinicId: inClinicIdId
            }
        }).then(function (num) {
            // Removed entries
            if (debug === true) {
                self.logger.debug("  --> removed clinic_id (id) " + inClinicIdId + " entries");
            }
            resolve("OK ");

        }).catch(function (delErr) {
            reject("CP associated Error - unable to clean up old data : " + delErr);
        });
    });
};


/**
 * Saves a an associated list of clinic catalog objects to the database tbl_clinic_catalog via promise
 * The input is an object of records - it return if successful the Id of the clinic_id associated
 *
 * @param clinic ID ID    (the ID in the tbl_clinic_profile of a clinic ID
 * @param catalog ID
 */
LoaderHelper.prototype.saveCPassociatedEntry = function(inClinicIdId, inCatalogId) {
    var self = this;

    return new Promise(function (resolve, reject) {
        self.logger.info("saveCPassociatedEntry for " + inClinicIdId + " : " + inCatalogId);

        // Save new list in database !

        self.models.ClinicCatalog.findOrCreate({
            where: {
                id: 0               // This to force to create always a new record !
            }, defaults: {
                clinicId: inClinicIdId,
                catalogId: inCatalogId
            }
        }).spread(function (data, created) {
            if (!created) {
                data.clinicId = inClinicIdId;
                data.catalogId = inCatalogId;

                data.save().then(function () {
                    if (debug === true) {
                        self.logger.debug("  --> updated associated entry for clinic_id (id) " + inClinicIdId + " - catalogId : " + data.catalogId);
                    }
                }).catch(function (err) {
                    reject("CP associated record - problem interacting with DB : " + err);
                });
            } else {
                if (debug === true) {
                    self.logger.debug("  --> created associated entry for clinic_id (id) " + inClinicIdId + " - catalogId : " + data.catalogId);
                }
            }
            resolve(inClinicIdId);
        }).catch(function (delErr) {
            reject("CP associated Error - unable to save new associated data : " + delErr);
        });
    });
};

/**
 * Saves a single Master Item record to the database tbl_master_item via promise
 * IMPORTANT ! It is assumed the data coming here are already vetted by m3-api during the
 * validation !!!!!!
 * It means no specific check on the data are performed here !  (note - FRES-192 removed the ndc)
 *
 * @param {Object} msg JSON from Http
 */
LoaderHelper.prototype.saveLoaderMT = function(msg) {
    var self = this;

    return new Promise(function (resolve, reject) {
        var itemIdId;
        var csv_line = msg.split(/,(?=(?:(?:[^"]*"){2})*[^"]*$)/);
        csv_line = csv_line || [];

        if (debug === true) {
            self.logger.debug("saveLoaderMT - csv line --> " + JSON.stringify(csv_line));
        }

        if (csv_line.length > 1) {
            try {
                // Building an object with the data received in input to update the tbl_master_item
                var masterItem = self.models.MasterItem.build({
                    sapMaterial:      parseInt(csv_line[rowMTPos.sapMaterial]),
                    description:      csv_line[rowMTPos.description].split('"').join(''),
                    cntInd:           csv_line[rowMTPos.cntInd].split('"').join('')  === 'X',
                    cntDescription:   csv_line[rowMTPos.cntDescription].split('"').join(''),
                    vendorMaterial:   csv_line[rowMTPos.vendorMaterial].split('"').join(''),
                    conv:             parseInt(csv_line[rowMTPos.conv].split('"').join('')),
                    countType:        csv_line[rowMTPos.countType].split('"').join(''),
                    alternates:       csv_line[rowMTPos.alternates].split('"').join(''),
                    message:          csv_line[rowMTPos.itemMessage].split('"').join(''),
                    unitPrice:        awcblcommon.formatPrice(csv_line[rowMTPos.unitPrice].split('"').join('')),
                    vendorId:         csv_line[rowMTPos.vendorId].split('"').join(''),
                    ndc: null,        // FRES-191 - not in use anymore
                    showInventoryInd: csv_line[rowMTPos.onHand].split('"').join('')  === 'X',
                    discontinued:     awcblcommon.formatDate(csv_line[rowMTPos.discontinued].split('"').join('')),
                    dateModified:     new Date()
                });

            } catch (e) {
                self.logger.error("TFG Formatting MT object for SAP Id " + csv_line[rowMTPos.sapMaterial] + " - Error : " + JSON.stringify(e));

                self.logger.error("TFG fields " );
                self.logger.error("TFG description : " + csv_line[rowMTPos.description] );
                self.logger.error("TFG vendorMaterial : " + csv_line[rowMTPos.vendorMaterial] );
                self.logger.error("TFG message : " + csv_line[rowMTPos.itemMessage] );
                self.logger.error("TFG unitPrice : " + csv_line[rowMTPos.unitPrice] );
                self.logger.error("TFG vendorId : " + csv_line[rowMTPos.vendorId] );

                reject("CAUTION !! MT - error while building MT object ! ");
            }

            if (isNaN(masterItem.conv)) {
                masterItem.conv = 1;
            }

            if (debug === true) {
                self.logger.debug("  Payload retrieved ");
                self.logger.debug("      sapMaterial : " + masterItem.sapMaterial);
                self.logger.debug("      description : " + masterItem.description);
                self.logger.debug("           cntInd : " + masterItem.cntInd);
                self.logger.debug("   cntDescription : " + masterItem.cntDescription);
                self.logger.debug("   vendorMaterial : " + masterItem.vendorMaterial);
                self.logger.debug("             conv : " + masterItem.conv);
                self.logger.debug("        countType : " + masterItem.countType);
                self.logger.debug("       alternates : " + masterItem.alternates);
                self.logger.debug("          message : " + masterItem.message);
                self.logger.debug("        unitPrice : " + masterItem.unitPrice);
                self.logger.debug("         vendorId : " + masterItem.vendorId);
                self.logger.debug("              ndc : " + masterItem.ndc);   // MUST BE FORCED NULL !
                self.logger.debug(" showInventoryInd : " + masterItem.showInventoryInd);
                self.logger.debug("     discontinued : " + masterItem.discontinued);
                self.logger.debug("     dateModified : " + masterItem.dateModified);
            }

            // Update the master item table - original way on old bulk load procedure
            // CAUTION ! TFG note
            // Apparently using the dataValues to update the master item record, ANY field set to null is NOT
            // included in the update.
            // This means that for example the field discontinued CAN NOT BE REVERSED from a bulk load.
            // If the field discontinued is set to null, because not present in the CSV record, it will NOT be
            // updated, thus if is discontinued in the db (date present) will continue to remains discontinued !
            // At the current date/release the ONLY way to reverse the discontinued record is going MANUALLY
            // on the DB and forcing null the discontinued field of the record !
            // Also from the UI there is not way to do !
            // Putting a warning log for documentation
            if (masterItem.discontinued === null) {
                self.logger.warn("WARNING! discontinued field for sap_material_id " + masterItem.sapMaterial + " is null. No actually update on this field in the DB");
            }

            // FRES-192 warning ! Just in case, should NEVER happens !
            if (masterItem.ndc !== null) {
                self.logger.warn("WARNING! ndc field for sap_material_id " + masterItem.sapMaterial + " is NOT null. Something very wrong here ");
            }

            self.models.MasterItem.findOrCreate({
                where: {
                    sapMaterial: masterItem.sapMaterial,
                    vendorMaterial: masterItem.vendorMaterial
                }, raw: true, paranoid: false
            }).spread(function (itemRecord, created) {
                masterItem.id = itemRecord.id;
                itemIdId = itemRecord.id;      // Save record ID

                self.models.MasterItem.update(masterItem.dataValues, { where: { id: masterItem.id } })
                    .then(function (count) {
                        if (debug === true) {
                            self.logger.debug("Master Item " + ((created) ? "created" : "updated") + " by ftp sync: " + JSON.stringify(masterItem));
                        }
                        // Bring back the table ID of the saved/updated record !
                        resolve(itemIdId);
                    }).catch(function (upderr) {
                    reject("MT Error handling table on update : " + upderr);
                });
            }).catch(function (err) {
                reject("MT Error handling table on findOrCreate : " + err);
            });

        } else {
            reject("CAUTION !! MT  - something wrong received ! ");
        }
    });
};

/**
 * Remove from the tbl_item_catalog all the entries related to a specific item ID
 *
 * @param item ID ID    (the ID in the tbl_master_item of a item ID
 */
LoaderHelper.prototype.removeMTassociated = function(inItemIdId) {
    var self = this;

    return new Promise(function (resolve, reject) {
        if (debug === true) {
            self.logger.debug("removeMTassociated for " + inItemIdId);
        }

        // Remove from the table all entries related to the clinicId received
        self.models.ItemCatalog.destroy({
            where: {
                itemId: inItemIdId
            }
        }).then(function (num) {
            // Removed entries
            if (debug === true) {
                self.logger.debug("  --> removed item_id (id) " + inItemIdId + " entries");
            }
            resolve("OK ");

        }).catch(function (delErr) {
            reject("MT associated Error - unable to clean up old data : " + delErr);
        });
    });
};


/**
 * Saves a an associated list of item catalog objects to the database tbl_item_catalog via promise
 * The input is an object of records - it return if successful the Id of the clinic_id associated
 *
 * @param item ID ID    (the ID in the tbl_master_item of a item ID
 * @param catalog ID
 */
LoaderHelper.prototype.saveMTassociatedEntry = function(inItemIdId, inCatalogId) {
    var self = this;

    return new Promise(function (resolve, reject) {
        self.logger.info("saveMTassociatedEntry for " + inItemIdId + " : " + inCatalogId);

        // Save new list in database !

        self.models.ItemCatalog.findOrCreate({
            where: {
                id: 0               // This to force to create always a new record !
            }, defaults: {
                itemId: inItemIdId,
                catalogId: inCatalogId
            }
        }).spread(function (data, created) {
            if (!created) {
                data.itemId = inItemIdId;
                data.catalogId = inCatalogId;

                data.save().then(function () {
                    if (debug === true) {
                        self.logger.debug("  --> updated associated entry for item_id (id) " + inItemIdId + " - catalogId : " + data.catalogId);
                    }
                }).catch(function (err) {
                    reject("MT associated record - problem interacting with DB - itemId : " + inItemIdId + " catalogId : " + inCatalogId + " Error : " + err);
                });
            } else {
                if (debug === true) {
                    self.logger.debug("  --> created associated entry for item_id (id) " + inItemIdId + " - catalogId : " + data.catalogId);
                }
            }
            resolve(inItemIdId);
        }).catch(function (delErr) {
            reject("MT associated Error - unable to save new associated data - itemId : " + inItemIdId + " catalogId : " + inCatalogId + " Error : " + delErr);
        });
    });
};

module.exports = LoaderHelper;
module.exports.debug = debug;
module.exports.rowMTPos = rowMTPos;
module.exports.rowCPPos = rowCPPos;
module.exports.rowCRPos = rowCRPos;
module.exports.rowPVPos = rowPVPos;