/**
 * Created by TFG on 07/15/2020.
 * Management for the Master Item loading data
 */
var app = require("../../child");

var LoaderHelper = require("../helpers/LoaderHelper");
var awcblcommon = require('../config/awcblcommon');

var expectedLoadingTime = 120000;     // Expected execution time (watchdog) in ms (default 2 minutes - 120 sec - 120000 ms)
var loadingCRwatchDog = null;         // Global variable to keep the loading timeout timer

var debug = LoaderHelper.debug;

// Object to deal with metadata
var CRmetadata = {
    transactionId: "N/A",     //    - transaction ID
    fileName: "--",           //    - name of the originating CSV file
    expectedNumRec: 0,        //    - number to record to be received (remove 1 when counting)
    rcvCounter: 0,            //    - counter of record received  (remember to add 1 when showing the value - counter from 0 not 1)
    dbCounter: 0,             //    - counter of record saved on DB ((remember to add 1 when showing the value - counter from 0 not 1)
    endLoad: false,           //    - flag to indicate the current load as ended
    endProc: false,           //    - flag to indicate the current processing as ended
    sendEndLoad: true,        //    - flag to be used to send out only ONE log notification about end loading
    sendEndProc: true,        //    - flag to be used to send out only ONE log notification about end processing
    incrementRcvCounter : function() {
        this.rcvCounter++;
        if (this.rcvCounter >=  (this.expectedNumRec -1)) {
            this.endLoad = true;
        }
    },
    incrementDbCounter : function() {
        this.dbCounter++;
        // Here the test is little bit different because the position of the incrementDbCounter
        // Starting from zero, when process the number 9 (the latest) it will be incremented by one and
        // then the test
        if (this.dbCounter >= this.expectedNumRec) {
            this.endProc = true;
        }
    }
};

// Create array of metadata objects
var metadata = [ CRmetadata ];

module.exports = function(winston, rabbit, models, config, cb) {
    // variable to hold starting time
    var started = Date.now();
    var logger = winston.loggers.get("main");

    // always setup your message handlers first
    rabbit.nackOnError();

    // this handler will handle messages sent from the publisher
    rabbit.handle({
        queue: 'topic-fresenius-loader-crq',
        type: 'loader.cr'
    }, function (msg) {
        var helper = new LoaderHelper(models, winston);

        logger.info(process.pid + ":LOADER CR POST Received:",
            JSON.stringify(msg.body,null, 4),
            "routingKey:", msg.fields.routingKey);

        // Metadata management - use the currentTransaction to monitor the load
        var currentTransaction = processMetadata(winston, msg.body);

        // Store the record on the DB via helper
        helper.saveLoaderCR(msg.body).then(response => {
            // Response - last record saved
            logger.info("Record " + currentTransaction.dbCounter + " saved : " + response);
            currentTransaction.incrementDbCounter();        // Increment saved record counter

            // Load a timer for about 2 minutes !
            // If it expires it means that no more records were received and not all the expected records
            // are arrived
            // If a previous timer is running, stop it
            if (loadingCRwatchDog) {
                if (debug === true) {
                    logger.debug("=======> Loading WatchDog ! Canceled for CR !");
                }
                clearTimeout(loadingCRwatchDog);       // Cancel timeout
            }

            loadingCRwatchDog = setTimeout(function () {
                logger.error("CR loading timeout EXPIRED !");

                // Write log to notify error happened
                awcblcommon.writeAwcblEntryLog(app,
                    currentTransaction.fileName,
                    currentTransaction.transactionId,
                    "ERROR",
                    "Missing expected records - " + expectedLoadingTime + " ms timer expired - Contact assistance").then(response => {
//                                    logger.debug("AWCBL log written : " + response);
                }, reject => {
                    logger.error("AWCBL log : " + reject);
                });
            }, expectedLoadingTime);

            if (currentTransaction.endProc === true && currentTransaction.sendEndProc === true) {
                // If a previous timer is running, stop it
                if (loadingCRwatchDog) {
                    if (debug === true) {
                        logger.debug("=======> Loading WatchDog ! Canceled for CR !");
                    }
                    clearTimeout(loadingCRwatchDog);       // Cancel timeout
                    loadingCRwatchDog = null;
                }

                // Write log to notify successfull end of operation
                awcblcommon.writeAwcblEntryLog(app,
                    currentTransaction.fileName,
                    currentTransaction.transactionId,
                    "PROCESSED",
                    "Processed " + currentTransaction.dbCounter + "  records").then(response => {
//                         logger.debug(response);
                }, reject => {
                    logger.error("AWCBL log : " + reject);
                });
                currentTransaction.sendEndProc = false;     // Disable sending other LOG messages about ending process
            }
            msg.ack();
        }, reject => {
            // Record not saved ! Ack message anyway to remove from the queue
            msg.ack();
            logger.error("Error from helper : " + reject);

            // Write log to notify error happened
            awcblcommon.writeAwcblEntryLog(app,
                currentTransaction.fileName,
                currentTransaction.transactionId,
                "ERROR",
                "Error loading : " + reject + " - Contact assistance").then(response => {
//                logger.debug(response);
            }, reject => {
                logger.error("AWCBL log : " + reject);
            });
        });

        // Shows in the log statistics about processing
        logger.info("Transaction " + currentTransaction.transactionId +
            " - expected " + currentTransaction.expectedNumRec +
            " - received " + currentTransaction.rcvCounter +
            " - stored " + currentTransaction.dbCounter +
            " - end load : " + currentTransaction.endLoad.toString() +
            " - end proc : " + currentTransaction.endProc.toString());

        if (currentTransaction.endLoad === true && currentTransaction.sendEndLoad === true) {
            logger.info("Transaction " + currentTransaction.transactionId + " ENDED !");

            // Write log to notify successfull end of operation
            awcblcommon.writeAwcblEntryLog(app,
                currentTransaction.fileName,
                currentTransaction.transactionId,
                "LOADED",
                "Loaded " + (currentTransaction.rcvCounter + 1) + " records").then(response => {
//                logger.debug(response);
            }, reject => {
                logger.error("AWCBL log : " + reject);
            });
            currentTransaction.sendEndLoad = false;     // Disable sending other LOG messages about ending loading
        }
    });

    logger.info(process.pid + ':Set up loader crq OK ');
    return require("../config/RabbitTopology.js")(rabbit, "loader.cr", config);
};

// Function to handle the metadata related to a load.
function processMetadata(winston, msg) {
    var path = require('path');
    var logger = winston.loggers.get("main");
    var inTransactionId;
    var inFileName;
    var inExpectedNumRec;

    // Extract metadata from input message - it is expected :
    //   csv_line[0] -> transactionId
    //   csv_line[1] -> file name
    //   csv_line[2] -> expected number of messages
    var csv_line = msg.split(/,(?=(?:(?:[^"]*"){2})*[^"]*$)/);
    csv_line = csv_line || [];

    // Retrieve metadata
    inTransactionId = csv_line[LoaderHelper.rowCRPos.transactionId].split(',').join('');
    inFileName = path.basename(csv_line[LoaderHelper.rowCRPos.fileName].split(',').join(''), ".csv") + ".csv";
    inExpectedNumRec = parseInt(csv_line[LoaderHelper.rowCRPos.expNumber].split(',').join(''), 10);

    // Check if the transaction ID exists on the array
    let currentTransaction = metadata.find(CRmetadata => CRmetadata.transactionId === inTransactionId);

    if (currentTransaction !== undefined) {
        currentTransaction.incrementRcvCounter();
    } else {
        logger.info("New transaction identified : " + inTransactionId);
        // Not found object - add one
        CRmetadata.transactionId  = inTransactionId;
        CRmetadata.fileName       = inFileName;
        CRmetadata.expectedNumRec = inExpectedNumRec;
        CRmetadata.rcvCounter     = 0;
        CRmetadata.dbCounter      = 0;
        CRmetadata.endLoad        = false;
        CRmetadata.endProc        = false;
        CRmetadata.sendEndLoad    = true;
        CRmetadata.sendEndProc    = true;

        metadata.push(CRmetadata);
        currentTransaction = metadata.find(CRmetadata => CRmetadata.transactionId === inTransactionId);
    }

    return currentTransaction;
}
